@@ -20,9 +20,13 @@
 # if (__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || (__GNUC__ > 4)
 #  pragma GCC diagnostic push
 # endif // (__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || (__GNUC__ > 4)
+// Ignore possibly unknown warning flags
+# pragma GCC diagnostic ignored "-Wpragmas"
 # pragma GCC diagnostic ignored "-Weffc++"
 # pragma GCC diagnostic ignored "-Wold-style-cast"
-#endif
+# pragma GCC diagnostic ignored "-Wunused-local-typedef"
+# pragma GCC diagnostic ignored "-Wunused-variable"
+#endif // __GNUG__
 #endif // ! HAVE_SYSTEM_ASIO
 
 #include "asio.hpp"
@@ -138,6 +142,19 @@ namespace gu
     // Error handling
     //
 
+    // Exclude some errors which are generated by the SSL library.
+    static inline bool exclude_ssl_error(const asio::error_code& ec)
+    {
+        switch (ERR_GET_REASON(ec.value()))
+        {
+        case SSL_R_SHORT_READ:
+            // Short read error seems to be generated quite frequently
+            // by SSL library, probably because broken connections.
+            return true;
+        }
+        return false;
+    }
+
     // Return low level error info for asio::error_code if available.
     static inline const std::string extra_error_info(const asio::error_code& ec)
     {
