@@ -11,6 +11,11 @@
 #include <iostream>
 #include <vector>
 
+static std::string const SCHED_OTHER_STR  ("other");
+static std::string const SCHED_FIFO_STR   ("fifo");
+static std::string const SCHED_RR_STR     ("rr");
+static std::string const SCHED_UNKNOWN_STR("unknown");
+
 static inline void parse_thread_schedparam(const std::string& param,
                                            int& policy,
                                            int& prio)
@@ -22,31 +27,14 @@ static inline void parse_thread_schedparam(const std::string& param,
         gu_throw_error(EINVAL) << "Invalid schedparam: " << param;
     }
 
-    if      (sv[0] == "other") policy = SCHED_OTHER;
-    else if (sv[0] == "fifo")  policy = SCHED_FIFO;
-    else if (sv[0] == "rr")    policy = SCHED_RR;
-    else gu_throw_error(EINVAL) << "Invalid scheduling policy: "
-                                << sv[0];
+    if      (sv[0] == SCHED_OTHER_STR) policy = SCHED_OTHER;
+    else if (sv[0] == SCHED_FIFO_STR)  policy = SCHED_FIFO;
+    else if (sv[0] == SCHED_RR_STR)    policy = SCHED_RR;
+    else gu_throw_error(EINVAL) << "Invalid scheduling policy: " << sv[0];
+
     prio = gu::from_string<int>(sv[1]);
 }
 
-static inline std::string print_thread_schedparam(int policy, int prio)
-{
-    std::ostringstream oss;
-    std::string policy_str;
-    switch (policy)
-    {
-    case SCHED_OTHER: policy_str = "other"  ; break;
-    case SCHED_FIFO:  policy_str = "fifo"   ; break;
-    case SCHED_RR:    policy_str = "rr"     ; break;
-    default:          policy_str = "unknown"; break;
-    }
-
-    oss << policy_str << ":" << prio;
-
-    return oss.str();
-}
-
 gu::ThreadSchedparam gu::ThreadSchedparam::system_default(SCHED_OTHER, 0);
 
 gu::ThreadSchedparam::ThreadSchedparam(const std::string& param)
@@ -64,6 +52,19 @@ gu::ThreadSchedparam::ThreadSchedparam(const std::string& param)
     }
 }
 
+void gu::ThreadSchedparam::print(std::ostream& os) const
+{
+    std::string policy_str;
+    switch (policy())
+    {
+    case SCHED_OTHER: policy_str = SCHED_OTHER_STR  ; break;
+    case SCHED_FIFO:  policy_str = SCHED_FIFO_STR   ; break;
+    case SCHED_RR:    policy_str = SCHED_RR_STR     ; break;
+    default:          policy_str = SCHED_UNKNOWN_STR; break;
+    }
+
+    os << policy_str << ":" << prio();
+}
 
 gu::ThreadSchedparam gu::thread_get_schedparam(pthread_t thd)
 {
@@ -86,9 +87,3 @@ void gu::thread_set_schedparam(pthread_t thd, const gu::ThreadSchedparam& sp)
         gu_throw_error(err) << "Failed to set thread schedparams " << sp;
     }
 }
-
-
-std::ostream& operator<<(std::ostream& os, const gu::ThreadSchedparam& sp)
-{
-    return (os << print_thread_schedparam(sp.policy(), sp.prio()));
-}
