@@ -548,9 +548,9 @@ void gcomm::GMCast::handle_connected(Proto* rp)
 
 void gcomm::GMCast::handle_established(Proto* est)
 {
-    log_debug << self_string() << " connection established to "
-              << est->remote_uuid() << " "
-              << est->remote_addr();
+    log_info << self_string() << " connection established to "
+             << est->remote_uuid() << " "
+             << est->remote_addr();
 
     if (is_evicted(est->remote_uuid()))
     {
@@ -615,7 +615,7 @@ void gcomm::GMCast::handle_established(Proto* est)
         i = remote_addrs_.insert_unique(
             make_pair(est->remote_addr(),
                       AddrEntry(gu::datetime::Date::now(),
-                                gu::datetime::Date::max(),
+                                gu::datetime::Date::now(),
                                 est->remote_uuid())));
     }
 
@@ -979,7 +979,6 @@ void gcomm::GMCast::reconnect()
         }
     }
 
-
     for (i = remote_addrs_.begin(); i != remote_addrs_.end(); i = i_next)
     {
         i_next = i, ++i_next;
@@ -1082,11 +1081,11 @@ void gcomm::GMCast::check_liveness()
             p->state() < Proto::S_FAILED &&
             p->tstamp() + peer_timeout_ < now)
         {
-            log_debug << self_string()
-                      << " connection to peer "
-                      << p->remote_uuid() << " with addr "
-                      << p->remote_addr()
-                      << " timed out";
+            log_info << self_string()
+                     << " connection to peer "
+                     << p->remote_uuid() << " with addr "
+                     << p->remote_addr()
+                     << " timed out, no messages seen in " << peer_timeout_;
             p->set_state(Proto::S_FAILED);
             handle_failed(p);
         }
@@ -1688,56 +1687,87 @@ void gcomm::GMCast::add_or_del_addr(const std::string& val)
 
 bool gcomm::GMCast::set_param(const std::string& key, const std::string& val)
 {
-    if (key == Conf::GMCastMaxInitialReconnectAttempts)
+    try
     {
-        max_initial_reconnect_attempts_ = gu::from_string<int>(val);
-        return true;
-    }
-    else if (key == Conf::GMCastPeerAddr)
-    {
-        try
+        if (key == Conf::GMCastMaxInitialReconnectAttempts)
         {
-            add_or_del_addr(val);
+            max_initial_reconnect_attempts_ = gu::from_string<int>(val);
+            return true;
         }
-        catch (gu::NotFound& nf)
+        else if (key == Conf::GMCastPeerAddr)
         {
-            gu_throw_error(EINVAL) << "invalid addr spec '" << val << "'";
-        }
-        catch (gu::NotSet& ns)
-        {
-            gu_throw_error(EINVAL) << "invalid addr spec '" << val << "'";
-        }
-        return true;
-    }
-    else if (key == Conf::GMCastIsolate)
-    {
-        isolate_ = gu::from_string<bool>(val);
-        log_info << "turning isolation "
-                 << (isolate_ == true ? "on" : "off");
-        if (isolate_ == true)
-        {
-            // delete all entries in proto map
-            ProtoMap::iterator pi, pi_next;
-            for (pi = proto_map_->begin(); pi != proto_map_->end(); pi = pi_next)
+            try
             {
-                pi_next = pi, ++pi_next;
-                erase_proto(pi);
+                add_or_del_addr(val);
             }
-            segment_map_.clear();
+            catch (gu::NotFound& nf)
+            {
+                gu_throw_error(EINVAL) << "invalid addr spec '" << val << "'";
+            }
+            catch (gu::NotSet& ns)
+            {
+                gu_throw_error(EINVAL) << "invalid addr spec '" << val << "'";
+            }
+            return true;
+        }
+        else if (key == Conf::GMCastIsolate)
+        {
+            isolate_ = gu::from_string<bool>(val);
+            log_info << "turning isolation "
+                     << (isolate_ == true ? "on" : "off");
+            if (isolate_ == true)
+            {
+                // delete all entries in proto map
+                ProtoMap::iterator pi, pi_next;
+                for (pi = proto_map_->begin(); pi != proto_map_->end();
+                     pi = pi_next)
+                {
+                    pi_next = pi, ++pi_next;
+                    erase_proto(pi);
+                }
+                segment_map_.clear();
+            }
+            return true;
+        }
+        else if (key == Conf::SocketRecvBufSize)
+        {
+            gu_trace(Conf::check_recv_buf_size(val));
+            conf_.set(key, val);
+
+            for (ProtoMap::iterator pi(proto_map_->begin());
+                 pi != proto_map_->end(); ++pi)
+            {
+                gu_trace(pi->second->socket()->set_option(key, val));
+                // erase_proto(pi++);
+            }
+            // segment_map_.clear();
+            // reconnect();
+            return true;
+        }
+        else if (key == Conf::GMCastGroup       ||
+                 key == Conf::GMCastListenAddr  ||
+                 key == Conf::GMCastMCastAddr   ||
+                 key == Conf::GMCastMCastPort   ||
+                 key == Conf::GMCastMCastTTL    ||
+                 key == Conf::GMCastTimeWait    ||
+                 key == Conf::GMCastPeerTimeout ||
+                 key == Conf::GMCastSegment)
+        {
+            gu_throw_error(EPERM) << "can't change value during runtime";
         }
-        return true;
     }
-    else if (key == Conf::GMCastGroup ||
-             key == Conf::GMCastListenAddr ||
-             key == Conf::GMCastMCastAddr ||
-             key == Conf::GMCastMCastPort ||
-             key == Conf::GMCastMCastTTL ||
-             key == Conf::GMCastTimeWait ||
-             key == Conf::GMCastPeerTimeout ||
-             key == Conf::GMCastSegment)
+    catch (gu::Exception& e)
     {
-        gu_throw_error(EPERM) << "can't change value for '"
-                              << key << "' during runtime";
+        GU_TRACE(e); throw;
     }
+    catch (std::exception& e)
+    {
+        gu_throw_error(EINVAL) << e.what();
+    }
+    catch (...)
+    {
+        gu_throw_error(EINVAL) << "exception";
+    }
+
     return false;
 }
